"""Provides ability to generate pre-formatted human-friendly output from services."""

from ioccheck.cli.formatters.formatter import Formatter
from ioccheck.cli.formatters.malwarebazaar import MalwareBazaarFormatter
from ioccheck.cli.formatters.shodan import ShodanFormatter
from ioccheck.cli.formatters.virustotal import VirusTotalFormatter
